#ifndef __PARTICLE__
#define __PARTICLE__


#include <ngl/Vector.h>
#include <ngl/Colour.h>
#include <ngl/TransformStack.h>

#ifdef LINUX
	#include <GL/gl.h>
	#include <GL/glut.h>
#endif
#ifdef DARWIN
	#include <unistd.h>
	#include <OpenGL/gl.h>
#endif





class Particle
{
public:
  // Constructor
	Particle(
	         ngl::Vector _pos,
	         ngl::Vector _dir,
	         ngl::Colour _c,
					 const std::string &_shaderName,
					 ngl::TransformStack *_transform
	        );
	// destructor
	virtual ~Particle();
	// method to draw the particle
	virtual void draw()=0;
	// methods to update the particle
	virtual void update()=0;
  inline void setMaxLife(int _l){m_maxLife=_l;}
protected :
  // Position of the Particle
  ngl::Vector m_pos;
  //Direction of the Particle
  ngl::Vector m_dir;
  // Colour of the Particle
  ngl::Colour m_colour;
	/// @brief the current life of the particle
  GLfloat m_life;
	/// @brief the max life of the partice
  GLfloat m_maxLife;
	/// @brief the name of the shader to use
	std::string m_shaderName;
	/// @brief a pointer to the global transform stack
	ngl::TransformStack *m_transform;
};



#endif

