#include "Sphere.h"
#include <cmath>
#include <ngl/Util.h>
#include <ngl/VBOPrimitives.h>
#include <ngl/Random.h>
#include <ngl/ShaderManager.h>

Sphere::Sphere(
                ngl::Vector _pos,
                ngl::Vector _dir,
                ngl::Colour _c,
                GLfloat _angle,
								const std::string &_shaderName,
								ngl::TransformStack *_transform

							  ) :
                  Particle(
                            _pos,
                            _dir,
                            _c,
													  _shaderName,
														_transform
                          )
{
  m_emitAngle=_angle;
	ngl::Random *rng=ngl::Random::instance();

  GLfloat theta=rng->randomPositiveNumber(m_emitAngle);
  GLfloat phi=rng->randomPositiveNumber(m_emitAngle);
  m_dir.m_x=sin(theta)*cos(phi);
  m_dir.m_y=sin(theta)*sin(phi);
  m_dir.m_z=cos(theta);
  m_dir.normalize();
	m_radius=rng->randomPositiveNumber(1.2)+0.1;
}


void Sphere::draw()
{

  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  (*shader)[m_shaderName]->use();
	shader->setShaderParam4f(m_shaderName,"Colour",m_colour.m_r,m_colour.m_g,m_colour.m_b,m_colour.m_a);

	m_transform->pushTransform();
	{
		m_transform->setPosition(m_pos);
		m_transform->setScale(m_radius,m_radius,m_radius);
		m_transform->loadGlobalAndCurrentMatrixToShader(m_shaderName,"ModelMatrix");

    // get an instance of the VBO primitives for drawing
    ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
    // draw the sphere
		prim->draw("sphere");

	}
	m_transform->popTransform();
}


void Sphere::update()
{
// more the particle by adding the Direction vector to the position
	
  m_pos+=m_dir;
  ++m_life;
  if(m_life > m_maxLife)
	{
    m_life=0.0;
    m_pos=0.0;
		ngl::Random *rng=ngl::Random::instance();

    m_maxLife=rng->randomPositiveNumber(80);
    GLfloat theta=rng->randomNumber(m_emitAngle);
    GLfloat phi=rng->randomNumber(m_emitAngle);
    m_dir.m_x=sin(theta)*cos(phi);
    m_dir.m_y=sin(theta)*cos(theta);
    m_dir.m_z=cos(theta);
    m_dir.normalize();
		m_radius=rng->randomPositiveNumber(1.2)+0.1;

	}
}


 Sphere::~Sphere()
{
   std::cout<<"Sphere dtor\n";
}

