#include "Teapot.h"
#include <cmath>
#include <ngl/VBOPrimitives.h>
#include <ngl/ShaderManager.h>
#include <ngl/Random.h>

Teapot::Teapot(
								ngl::Vector _pos,
								ngl::Vector _dir,
								ngl::Colour _c,
								const std::string &_shaderName,
								ngl::TransformStack *_transform

								) :
									 Particle(
														 _pos,
														 _dir,
														 _c,
														 _shaderName,
														 _transform
														)
{
	ngl::Random *rng=ngl::Random::instance();

	m_rotX=rng->randomNumber(360);
	m_rotY=rng->randomNumber(360);
	m_rotZ=rng->randomNumber(360);

	m_rotUpdateX=rng->randomNumber(10);
	m_rotUpdateY=rng->randomNumber(10);
	m_rotUpdateZ=rng->randomNumber(10);

}


void Teapot::draw()
{
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  (*shader)[m_shaderName]->use();
	shader->setShaderParam4f(m_shaderName,"Colour",m_colour.m_r,m_colour.m_g,m_colour.m_b,m_colour.m_a);

	m_transform->pushTransform();
	{
		m_transform->setPosition(m_pos);
		m_transform->setRotation(m_rotX,m_rotY,m_rotZ);
		m_transform->loadGlobalAndCurrentMatrixToShader(m_shaderName,"ModelMatrix");

    // get an instance of the VBO primitives for drawing
    ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
    // draw the sphere
		prim->draw("teapot");

	}
	m_transform->popTransform();
}


void Teapot::update()
{
// more the particle by adding the Direction vector to the position
	ngl::Random *rng=ngl::Random::instance();

  m_pos+=m_dir;
  ++m_life;
	m_rotX+=m_rotUpdateX;
	m_rotY+=m_rotUpdateY;
	m_rotZ+=m_rotUpdateZ;

  if(m_life > m_maxLife)
	{
    m_life=0.0;
    m_pos=0.0;
		m_rotX=rng->randomNumber(360);
		m_rotUpdateX=rng->randomPositiveNumber(10);
		m_rotY=rng->randomNumber(360);
		m_rotUpdateY=rng->randomPositiveNumber(10);
		m_rotZ=rng->randomNumber(360);
		m_rotUpdateZ=rng->randomPositiveNumber(10);
	}
}


Teapot::~Teapot()
{
  std::cout<<"Teapot dtor\n";

}

